# -*- coding:utf-8 -*-

from OCStyleMaster.models.blocks.base.blockBase import *
import os
from OCStyleMaster.globalData import *


class File(BlockBase):

    def __init__(self, filePath):
        f = open(filePath, "r")
        text = f.read()
        f.close()
        super(File, self).__init__(text)
        self.path = filePath
        _, ext = os.path.splitext(self.path)
        self.ext = ext
        self.filename = os.path.basename(self.path)
        self.file = self
        self.type = Block.file
        self.range = Range(0,len(self.text))
        self.lineCount = 0
        self.linePoses = self.__get_line_poses()
        self.errors = []


    def analyze(self):
        """
        分析主函数
        :return:
        """
        for child in self.children:
            child.analyze()



    def __analyze_line(self):
        lines = self.content().split("\n")
        index = 0
        for line in lines:
            length = len(line)
            if 80 < length and length <= 100:
                err = StyleError(index+1, 1, ErrorType.suggest, "行过长，超过80字节")
                self.add_error_obj(err)
            elif length > 100:
                err = StyleError(index+1, 1, ErrorType.warn, "行过长，超过100字节")
                self.add_error_obj(err)
            else:
                pass
            index+=1

    def __get_line_poses(self):
        start = 0
        ret = []
        while True:
            pos = self.text.find("\n",start)
            if pos < 0:
                break
            ret.append(pos)
            start = pos+1
        return ret


    def ext(self):
        """
        后缀
        :return:
        """
        return self.ext


    def output_all_errors(self):
        """
        输出 错误
        :return:
        """
        errors = sorted(self.errors,key = lambda obj:obj.line)
        if GlobalData().fileHandler is None:
            print("[{}]".format(self.filename))
            for e in errors:
                print(e)
        else:
            h = GlobalData().fileHandler
            file = "[{}]\n".format(self.filename)
            h.write(file)
            for e in errors:
                string = str(e) + "\n"
                h.write(string)
            h.flush()

